/*
 * stm32l4r5xx_uart.c
 *
 *  Created on: Apr 23, 2021
 *      Author: pratham_ovalekar
 */


#include "stm32l4r5xx_uart.h"
#include "common.h"

/* Includes for freeRTOS queues. */
#include "FreeRTOS.h"
#include "queue.h"

/* Queues used to hold received characters */
static QueueHandle_t USART3_RxQueue;

extern void USART3_SerialISRCreateQueues( unsigned portBASE_TYPE uxQueueLength,
		QueueHandle_t *pxRxedChars );


/*!
 *  \brief USART Peripheral Clock Setup function
 *
 *  \param pUSARTx
 *  Base Address of the USART
 *
 *  \param EnorDi
 *  ENABLE or DISABLE macros
 *
 */
void USART_PeriClockControl( USART_TypeDef *pUSARTx, uint8_t EnorDi )
{
	if ( EnorDi  == ENABLE )
		{
			if( pUSARTx == USART1 )
				USART1_PCLK_EN();
			else if( pUSARTx == USART2 )
				USART2_PCLK_EN();
			else if( pUSARTx == USART3 )
				USART3_PCLK_EN();
			else if( pUSARTx == UART4 )
				UART4_PCLK_EN();
			else if( pUSARTx == UART5 )
				UART5_PCLK_EN();
		}
		else if( EnorDi == DISABLE )
		{
			if( pUSARTx == USART1 )
				USART1_PCLK_DI();
			else if( pUSARTx == USART2 )
				USART2_PCLK_DI();
			else if( pUSARTx == USART3 )
				USART3_PCLK_DI();
			else if( pUSARTx == UART4 )
				UART4_PCLK_DI();
			else if( pUSARTx == UART5 )
				UART5_PCLK_DI();
		}

}


/*!
 *  \brief Initialize USARTx peripheral
 *
 *  \param pUSARTHandle
 *  Handle to USARTx Inti/Config structure
 *
 */
void USART_Init( USART_Handle_t *pUSARTHandle )
{
	uint32_t tempRegCR1;
	uint32_t tempRegCR2;
	uint32_t tempRegCR3;

	tempRegCR1 = 0;
	tempRegCR2 = 0;
	tempRegCR3 = 0;


	// enable the Clock for given USART peripheral
	USART_PeriClockControl( pUSARTHandle->pUSARTx, ENABLE );

	/************************* Configuration of CR1 ***************************/

	// Configure UART Mode
	// Configure Transmit Bit(3) and Receive Bit(2) in the CR1 Register(USART_CR1)
	switch( pUSARTHandle->USART_Config.USART_Mode )
	{
		case USART_MODE_ONLY_TX:
			tempRegCR1 |= ( 1 << USART_CR1_TE_Pos );
			break;


		case USART_MODE_ONLY_RX:
			tempRegCR1 |= ( 1 << USART_CR1_RE_Pos );
			break;

		case USART_MODE_TXRX:
			tempRegCR1 |= ( 1 << USART_CR1_RE_Pos );
			tempRegCR1 |= ( 1 << USART_CR1_TE_Pos );
			break;
	}


	// Configure word length
	// The word length can be set to 7, 8 or 9 bits, by programming the M bits
	// ( M1:  bit 28 & M0: bit 12 ) in the USART_CR1 register
	//  M[1:0] = ‘00’: 1 start bit, 8 Data bits, n Stop bit
	//  M[1:0] = ‘01’: 1 start bit, 9 Data bits, n Stop bit
	//  M[1:0] = ‘10’: 1 start bit, 7 Data bits, n Stop bit
	switch( pUSARTHandle->USART_Config.USART_WordLength )
	{
		case USART_WORDLEN_8BITS:
			tempRegCR1 &= ~( 1 << USART_CR1_M1_Pos );
			tempRegCR1 &= ~( 1 << USART_CR1_M0_Pos );
			break;

		case USART_WORDLEN_9BITS:
			tempRegCR1 &= ~( 1 << USART_CR1_M1_Pos );
			tempRegCR1 |= ( 1 << USART_CR1_M0_Pos );
			break;

		case USART_WORDLEN_7BITS:
			tempRegCR1 |= ( 1 << USART_CR1_M1_Pos );
			tempRegCR1 &= ~( 1 << USART_CR1_M0_Pos );
			break;

	}


	// Configure parity control→ no, even or odd
	// Configure PCE: Parity control enable Bit(10) &
	//	Parity selection bit(9) :: ( 0: Even parity ) (	1: Odd parity )
	//	in the CR1 Register(USART_CR1)
	switch( pUSARTHandle->USART_Config.USART_ParityControl )
	{
		case USART_PARITY_DISABLE:
			tempRegCR1 &= ~( 1 << USART_CR1_PCE_Pos );
			break;


		case USART_PARITY_EN_EVEN:
			tempRegCR1 |= ( 1 << USART_CR1_PCE_Pos );
			tempRegCR1 &= ~( 1 << USART_CR1_PS_Pos );
			break;

		case USART_PARITY_EN_ODD:
			tempRegCR1 |= ( 1 << USART_CR1_PCE_Pos );
			tempRegCR1 |= ( 1 << USART_CR1_PS_Pos );
			break;
	}

	//Program the CR1 register
	pUSARTHandle->pUSARTx->CR1 |= tempRegCR1;



	/************************* Configuration of CR2 ***************************/

	// Configure number of stop bits (USART_CR2)
	// USART_CR2 : Bits 13:12 STOP[1:0]
	switch( pUSARTHandle->USART_Config.USART_NumOfStopBits )
	{
		case USART_STOPBITS_1:
			tempRegCR2 &= ~( 1 << 13 );
			tempRegCR2 &= ~( 1 << USART_CR2_STOP_Pos );
			break;

		case USART_STOPBITS_0_5:
			tempRegCR2 &= ~( 1 << 13 );
			tempRegCR2 |= ( 1 << USART_CR2_STOP_Pos );
			break;

		case USART_STOPBITS_2:
			tempRegCR2 |= ( 1 << 13 );
			tempRegCR2 &= ~( 1 << USART_CR2_STOP_Pos );
			break;

		case USART_STOPBITS_1_5:
			tempRegCR2 |= ( 1 << 13 );
			tempRegCR2 |= ( 1 << USART_CR2_STOP_Pos );
			break;
	}

	//Program the CR2 register
	pUSARTHandle->pUSARTx->CR2 |= tempRegCR2;

	/************************* Configuration of CR3 ***************************/

	// Configure hardware flow control (USART_CR3)
	//	Bit 9 CTSE: CTS enable
	//	Bit 8 RTSE: RTS enable
	switch( pUSARTHandle->USART_Config.USART_HWFlowControl )
	{
		case USART_HW_FLOW_CTRL_NONE:
			tempRegCR3 &= ~( 1 << USART_CR3_CTSE_Pos );
			tempRegCR3 &= ~( 1 << USART_CR3_RTSE_Pos );
			break;


		case USART_HW_FLOW_CTRL_RTS:
			tempRegCR3 &= ~( 1 << USART_CR3_CTSE_Pos );
			tempRegCR3 |= ( 1 << USART_CR3_RTSE_Pos );
			break;

		case USART_HW_FLOW_CTRL_CTS:
			tempRegCR3 |= ( 1 << USART_CR3_CTSE_Pos );
			tempRegCR3 &= ~( 1 << USART_CR3_RTSE_Pos );
			break;

		case USART_HW_FLOW_CTRL_CTS_RTS:
			tempRegCR3 |= ( 1 << USART_CR3_CTSE_Pos );
			tempRegCR3 |= ( 1 << USART_CR3_RTSE_Pos );
			break;
	}
	//Program the CR2 register
	pUSARTHandle->pUSARTx->CR3 |= tempRegCR3;


	// Configure baud rate
	// USART baud rate register (USART_BRR)
	// This code will only set the baud rate to 9600
	// This is not modular. It is only resposnbile
	// Assumption is that the clock frequency to the USART is 4Mhz
	// We will have to modify the code when we have an actual crystal and the
	// corresponding clock frequency
	switch( pUSARTHandle->USART_Config.USART_Baud )
	{
		case USART_STD_BAUD_9600:
			pUSARTHandle->pUSARTx->BRR = 0x000001A0;
			break;

		case USART_STD_BAUD_19200:
			pUSARTHandle->pUSARTx->BRR = 0x000000D0;
			break;

		case USART_STD_BAUD_115200:
			pUSARTHandle->pUSARTx->BRR = 0x00000032;
			break;
	}



}



/*!
 *  \brief USART Peripheral Control (Enable or Disable the USART)
 *
 *  \param pUSARTx
 *  Base Address of the USART
 *
 *  \param EnorDi
 *  ENABLE or DISABLE macros
 *
 */
void USART_PeripheralControl(USART_TypeDef *pUSARTx, uint8_t EnOrDi)
{
	if( EnOrDi == ENABLE )
	{
		pUSARTx->CR1 |= ( 1 << USART_CR1_UE_Pos );
	}
	else
	{
		pUSARTx->CR1 &= ~( 1 << USART_CR1_UE_Pos );
	}

}



/*!
 *  \brief Get the status of the Flag from the USART ISR register
 *  USART Interrupt and Status register
 *
 *  \param pUSARTx
 *  Base Address of the USART
 *
 *  \param FlagName
 *  Location of the Flag in the register
 *  example : (0x1UL << USART_ISR_TC_Pos)
 *
 */
uint8_t USART_GetFlagStatus(USART_TypeDef *pUSARTx , uint32_t FlagName)
{
    if( pUSARTx->ISR & FlagName )
    {
    	return SET;
    }

   return RESET;
}



/*!
 *  \brief Clear the status of the Flag from the USART ISR register
 *  USART Interrupt and Status register
 *
 *  \param pUSARTx
 *  Base Address of the USART
 *
 *  \param FlagName
 *  Location of the Flag in the register
 *  example : (0x1UL << USART_ISR_TC_Pos)
 *
 */
void USART_ClearFlag(USART_TypeDef *pUSARTx, uint16_t StatusFlagName)
{
	pUSARTx->ISR &= ~( StatusFlagName);

}


void USART_SendData(USART_Handle_t *pUSARTHandle, uint8_t *pTxBuffer, uint32_t Len)
{

	uint16_t *pdata;

   //Loop over until "Len" number of bytes are transferred
	for(uint32_t i = 0 ; i < Len; i++)
	{
		//Implement the code to wait until TXE flag is set in the SR
		while(! USART_GetFlagStatus(pUSARTHandle->pUSARTx,USART_FLAG_TXE));

		//Check the USART_WordLength item for 9BIT or 8BIT in a frame
		if(pUSARTHandle->USART_Config.USART_WordLength == USART_WORDLEN_9BITS)
		{
			//if 9BIT load the DR with 2bytes masking  the bits other than first 9 bits
			pdata = (uint16_t*) pTxBuffer;
			pUSARTHandle->pUSARTx->TDR = (*pdata & (uint16_t)0x01FF);

			//check for USART_ParityControl
			if(pUSARTHandle->USART_Config.USART_ParityControl == USART_PARITY_DISABLE)
			{
				//No parity is used in this transfer , so 9bits of user data will be sent
				//Implement the code to increment pTxBuffer twice
				pTxBuffer++;
				pTxBuffer++;
			}
			else
			{
				//Parity bit is used in this transfer . so 8bits of user data will be sent
				//The 9th bit will be replaced by parity bit by the hardware
				pTxBuffer++;
			}
		}
		else
		{
			//This is 8bit data transfer
			pUSARTHandle->pUSARTx->TDR = (*pTxBuffer  & (uint8_t)0xFF);

			//Implement the code to increment the buffer address
			pTxBuffer++;
		}
	}

	//Implement the code to wait till TC flag is set in the SR
	while( ! USART_GetFlagStatus(pUSARTHandle->pUSARTx,USART_FLAG_TC));




}



void USART_ReceiveData(USART_Handle_t *pUSARTHandle, uint8_t *pRxBuffer, uint32_t Len)
{
   //Loop over until "Len" number of bytes are transferred
	for(uint32_t i = 0 ; i < Len; i++)
	{
		//Implement the code to wait until RXNE flag is set in the SR
		while(! USART_GetFlagStatus(pUSARTHandle->pUSARTx,USART_FLAG_RXNE));

		//Check the USART_WordLength to decide whether we are going to receive 9bit of data in a frame or 8 bit
		if(pUSARTHandle->USART_Config.USART_WordLength == USART_WORDLEN_9BITS)
		{
			//We are going to receive 9bit data in a frame

			//Now, check are we using USART_ParityControl control or not
			if(pUSARTHandle->USART_Config.USART_ParityControl == USART_PARITY_DISABLE)
			{
				//No parity is used , so all 9bits will be of user data

				//read only first 9 bits so mask the DR with 0x01FF
				*((uint16_t*) pRxBuffer) = (pUSARTHandle->pUSARTx->RDR  & (uint16_t)0x01FF);

				//Now increment the pRxBuffer two times
				pRxBuffer++;
				pRxBuffer++;
			}
			else
			{
				//Parity is used, so 8bits will be of user data and 1 bit is parity
				 *pRxBuffer = (pUSARTHandle->pUSARTx->RDR  & (uint8_t)0xFF);
				 pRxBuffer++;
			}
		}
		else
		{
			//We are going to receive 8bit data in a frame

			//Now, check are we using USART_ParityControl control or not
			if(pUSARTHandle->USART_Config.USART_ParityControl == USART_PARITY_DISABLE)
			{
				//No parity is used , so all 8bits will be of user data

				//read 8 bits from DR
				 *pRxBuffer = (uint8_t) (pUSARTHandle->pUSARTx->RDR  & (uint8_t)0xFF);
			}

			else
			{
				//Parity is used, so , 7 bits will be of user data and 1 bit is parity

				//read only 7 bits , hence mask the DR with 0X7F
				 *pRxBuffer = (uint8_t) (pUSARTHandle->pUSARTx->RDR  & (uint8_t)0x7F);

			}

			//Now , increment the pRxBuffer
			pRxBuffer++;
		}
	}

}



/*!
 *  \brief
 *
 */
void USART2_IRQConfigEnable( void )
{
	USART2->CR1 |= ( 1 << USART_CR1_RXNEIE_RXFNEIE_Pos );;
	/*
	NVIC_SetPriorityGrouping
	NVIC_EncodePriority
	NVIC_SetPriority
	*/
	NVIC_SetPriority(USART2_IRQn, 6);
	// Enable
	NVIC_EnableIRQ( USART2_IRQn );
}


void USART3_IRQConfigEnable( void )
{
	USART3->CR1 |= ( 1 << USART_CR1_RXNEIE_RXFNEIE_Pos );;
	/*
	NVIC_SetPriorityGrouping
	NVIC_EncodePriority
	NVIC_SetPriority
	*/
	//
	NVIC_SetPriority(USART3_IRQn, 6);
	// Enable
	NVIC_EnableIRQ( USART3_IRQn );
}


/*!
 *  \brief Create a RX queue for receiving messages received from the UART3
 *  Bluetooth.
 *  UART_BLUETOOTH_QUEUE_SIZE: value on common.h
 *  This function needs to pass the pointer to the queue.
 *
 */
void USART3_queueInit( void )
{
	USART3_SerialISRCreateQueues( UART_BLUETOOTH_QUEUE_SIZE, &USART3_RxQueue );
	return;
}



bool USART3_BluetoothRx( uint8_t *pcRxedChar )
{

	uint8_t testRx;
	if( xQueueReceive( USART3_RxQueue, &testRx, 0) )
	{
		*pcRxedChar = testRx;
		return true;
	}
	else
	{
		return false;
	}
}


void USART3_queueReset( void )
{
	xQueueReset( USART3_RxQueue );
}


void sendDataOnQueue( void )
{
	//BaseType_t xHigherPriorityTaskWoken;

	uint16_t arr = 80;
	static int indexVal = 0;

	uint16_t arrChar[11] = {80,82,65,84,72,65,77,69,83,72,32};


	if( indexVal >10 )
		indexVal = 0;
	arr = arrChar[indexVal];

	xQueueSend( USART3_RxQueue, &arr, ( TickType_t ) 0 );

	indexVal = indexVal + 1;
}




